@@ -68,6 +68,67 @@
  * a #ClutterAnimation instance and animate an actor between its current
  * state and the specified final state.
  *
+ * <refsect2 id="clutter-AnimationMode-Script">
+ *   <title>Defining ClutterAnimationMode inside ClutterScript</title>
+ *   <para>When defining a #ClutterAnimation inside a ClutterScript
+ *   file or string the #ClutterAnimation:mode can be defined either
+ *   using the #ClutterAnimationMode enumeration values through their
+ *   "nick" (the short string used inside #GEnumValue), their numeric
+ *   id, or using the following strings:</para>
+ *   <variablelist>
+ *     <varlistentry>
+ *       <term>easeInQuad, easeOutQuad, easeInOutQuad</term>
+ *       <listitem><para>Corresponding to the quadratic easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInCubic, easeOutCubic, easeInOutCubic</term>
+ *       <listitem><para>Corresponding to the cubic easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInQuart, easeOutQuart, easeInOutQuart</term>
+ *       <listitem><para>Corresponding to the quartic easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInQuint, easeOutQuint, easeInOutQuint</term>
+ *       <listitem><para>Corresponding to the quintic easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInSine, easeOutSine, easeInOutSine</term>
+ *       <listitem><para>Corresponding to the sine easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInExpo, easeOutExpo, easeInOutExpo</term>
+ *       <listitem><para>Corresponding to the exponential easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInCirc, easeOutCirc, easeInOutCirc</term>
+ *       <listitem><para>Corresponding to the circular easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInElastic, easeOutElastic, easeInOutElastic</term>
+ *       <listitem><para>Corresponding to the overshooting elastic
+ *       easing modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInBack, easeOutBack, easeInOutBack</term>
+ *       <listitem><para>Corresponding to the overshooting cubic
+ *       easing modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInBounce, easeOutBounce, easeInOutBounce</term>
+ *       <listitem><para>Corresponding to the bouncing easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *   </variablelist>
+ * </refsect2>
+ *
  * #ClutterAnimation is available since Clutter 1.0
  */
 
@@ -85,6 +146,8 @@
 #include "clutter-enum-types.h"
 #include "clutter-interval.h"
 #include "clutter-private.h"
+#include "clutter-scriptable.h"
+#include "clutter-script-private.h"
 
 enum
 {
@@ -125,7 +188,22 @@ static guint animation_signals[LAST_SIGNAL] = { 0, };
 
 static GQuark quark_object_animation = 0;
 
-G_DEFINE_TYPE (ClutterAnimation, clutter_animation, G_TYPE_OBJECT);
+static void clutter_scriptable_init (ClutterScriptableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (ClutterAnimation, clutter_animation, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
+                                                clutter_scriptable_init));
+
+static void
+on_actor_dispose (gpointer  user_data,
+                  GObject  *actor_pointer)
+{
+  ClutterAnimation *self = user_data;
+
+  if (self->priv->object == actor_pointer)
+    g_object_unref (self);
+}
+
 
 static void
 clutter_animation_real_completed (ClutterAnimation *self)
@@ -168,6 +246,7 @@ clutter_animation_real_completed (ClutterAnimation *self)
   if (animation == self)
     {
       g_object_set_qdata (priv->object, quark_object_animation, NULL);
+      g_object_weak_unref (priv->object, on_actor_dispose, self);
       g_object_unref (animation);
     }
 }
@@ -303,6 +382,49 @@ clutter_animation_get_property (GObject    *gobject,
     }
 }
 
+static gboolean
+clutter_animation_parse_custom_node (ClutterScriptable *scriptable,
+                                     ClutterScript     *script,
+                                     GValue            *value,
+                                     const gchar       *name,
+                                     JsonNode          *node)
+{
+  if (strncmp (name, "mode", 4) == 0)
+    {
+      if (json_node_get_node_type (node) != JSON_NODE_VALUE)
+        return FALSE;
+
+      g_value_init (value, G_TYPE_ULONG);
+
+      if (json_node_get_value_type (node) == G_TYPE_INT64)
+        {
+          g_value_set_ulong (value, json_node_get_int (node));
+          return TRUE;
+        }
+      else if (json_node_get_value_type (node) == G_TYPE_STRING)
+        {
+          const gchar *str = json_node_get_string (node);
+          gulong mode = CLUTTER_LINEAR;
+
+          mode = clutter_script_resolve_animation_mode (str);
+          g_value_set_ulong (value, mode);
+
+          return TRUE;
+        }
+      else
+        g_warning ("Expected an integer id or a string id for "
+                   "the ClutterAnimation mode property");
+    }
+
+  return FALSE;
+}
+
+static void
+clutter_scriptable_init (ClutterScriptableIface *iface)
+{
+  iface->parse_custom_node = clutter_animation_parse_custom_node;
+}
+
 static void
 clutter_animation_class_init (ClutterAnimationClass *klass)
 {
@@ -545,8 +667,7 @@ clutter_animation_validate_bind (ClutterAnimation *animation,
       return NULL;
     }
 
-  if (!g_value_type_compatible (G_PARAM_SPEC_VALUE_TYPE (pspec),
-                                argtype))
+  if (!g_value_type_compatible (G_PARAM_SPEC_VALUE_TYPE (pspec), argtype))
     {
       g_warning ("Cannot bind property '%s': the interval value of "
                  "type '%s' is not compatible with the property value "
@@ -556,6 +677,7 @@ clutter_animation_validate_bind (ClutterAnimation *animation,
                  g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
       return NULL;
     }
+
   return pspec;
 }
 
@@ -632,8 +754,7 @@ clutter_animation_bind (ClutterAnimation *animation,
   priv = animation->priv;
 
   type = G_VALUE_TYPE (final);
-  pspec = clutter_animation_validate_bind (animation, property_name,
-                                           type);
+  pspec = clutter_animation_validate_bind (animation, property_name, type);
   if (pspec == NULL)
     return NULL;
 
@@ -766,6 +887,59 @@ clutter_animation_update_interval (ClutterAnimation *animation,
   clutter_animation_update_property_internal (animation, pspec, interval);
 }
 
+/**
+ * clutter_animation_update:
+ * @animation: a #ClutterAnimation
+ * @property_name: name of the property
+ * @final: The final value of the property
+ *
+ * Updates the @final value of the interval for @property_name
+ *
+ * Return value: (transfer none): The animation itself.
+ *
+ * Since: 1.0
+ */
+ClutterAnimation *
+clutter_animation_update (ClutterAnimation *animation,
+                          const gchar      *property_name,
+                          const GValue     *final)
+{
+  ClutterAnimationPrivate *priv;
+  ClutterInterval *interval;
+
+  g_return_val_if_fail (CLUTTER_IS_ANIMATION (animation), NULL);
+  g_return_val_if_fail (property_name != NULL, NULL);
+  g_return_val_if_fail (final != NULL, NULL);
+  g_return_val_if_fail (G_VALUE_TYPE (final) != G_TYPE_INVALID, NULL);
+
+  priv = animation->priv;
+
+  interval = clutter_animation_get_interval (animation, property_name);
+  if (interval == NULL)
+    {
+      g_warning ("Cannot update property '%s': the animation has "
+                 "no bound property with that name",
+                 property_name);
+      return NULL;
+    }
+
+  if (!g_value_type_compatible (G_VALUE_TYPE (final),
+                                clutter_interval_get_value_type (interval)))
+    {
+      g_warning ("Cannot update property '%s': the interval value of "
+                 "type '%s' is not compatible with the property value "
+                 "of type '%s'",
+                 property_name,
+                 g_type_name (clutter_interval_get_value_type (interval)),
+                 g_type_name (G_VALUE_TYPE (final)));
+      return NULL;
+    }
+
+  clutter_interval_set_final_value (interval, final);
+
+  return animation;
+}
+
 /**
  * clutter_animation_get_interval:
  * @animation: a #ClutterAnimation
@@ -1563,6 +1737,39 @@ clutter_animation_setupv (ClutterAnimation    *animation,
     }
 }
 
+static const struct
+{
+  const gchar *name;
+  GConnectFlags flags;
+} signal_prefixes[] =
+  {
+    { "::", 0 },
+    { "-swapped::", G_CONNECT_SWAPPED },
+    { "-after::", G_CONNECT_AFTER },
+    { "-swapped-after::", G_CONNECT_SWAPPED | G_CONNECT_AFTER }
+  };
+
+static gboolean
+clutter_animation_has_signal_prefix (const gchar *property_name,
+                                     GConnectFlags *flags,
+                                     int *offset)
+{
+  int i;
+
+  if (!g_str_has_prefix (property_name, "signal"))
+    return FALSE;
+
+  for (i = 0; i < G_N_ELEMENTS (signal_prefixes); i++)
+    if (g_str_has_prefix (property_name + 6, signal_prefixes[i].name))
+      {
+        *offset = strlen (signal_prefixes[i].name) + 6;
+        *flags = signal_prefixes[i].flags;
+        return TRUE;
+      }
+
+  return FALSE;
+}
+
 static void
 clutter_animation_setup_valist (ClutterAnimation *animation,
                                 const gchar      *first_property_name,
@@ -1581,46 +1788,20 @@ clutter_animation_setup_valist (ClutterAnimation *animation,
       GValue final = { 0, };
       gchar *error = NULL;
       gboolean is_fixed = FALSE;
+      GConnectFlags flags;
+      int offset;
 
-      if (g_str_has_prefix (property_name, "signal::"))
+      if (clutter_animation_has_signal_prefix (property_name,
+                                               &flags,
+                                               &offset))
         {
-          const gchar *signal_name = property_name + 8;
+          const gchar *signal_name = property_name + offset;
           GCallback callback = va_arg (var_args, GCallback);
           gpointer  userdata = va_arg (var_args, gpointer);
 
-          g_signal_connect (animation, signal_name, callback, userdata);
-        }
-      else if (g_str_has_prefix (property_name, "signal-"))
-        {
-          GCallback callback = va_arg (var_args, GCallback);
-          gpointer  userdata = va_arg (var_args, gpointer);
-          const gchar *signal_name;
-          GConnectFlags flags;
-
-          if (g_str_has_prefix (property_name, "signal-after::"))
-            {
-              signal_name = property_name + 14;
-              flags = G_CONNECT_AFTER;
-            }
-          else if (g_str_has_prefix (property_name, "signal-swapped::"))
-            {
-              signal_name = property_name + 16;
-              flags = G_CONNECT_SWAPPED;
-            }
-          else
-            {
-              g_warning ("Unable to connect to '%s': the valid signal "
-                         "modifiers are 'signal-swapped::' and "
-                         "'signal-swapped::'",
-                         property_name);
-              break;
-            }
-
           g_signal_connect_data (animation, signal_name,
-                                 callback,
-                                 userdata,
-                                 NULL,
-                                 flags);
+                                 callback, userdata,
+                                 NULL, flags);
         }
       else
         {
@@ -1671,6 +1852,7 @@ animation_create_for_actor (ClutterActor *actor)
       animation = clutter_animation_new ();
       clutter_animation_set_object (animation, object);
       g_object_set_qdata (object, quark_object_animation, animation);
+      g_object_weak_ref (object, on_actor_dispose, animation);
 
       CLUTTER_NOTE (ANIMATION,
                     "Created new Animation [%p] for actor [%p]",
