@@ -28,6 +28,62 @@
  *
  * Functions to retrieve various global Clutter resources and other utility
  * functions for mainloops, events and threads
+ *
+ * <refsect2 id="clutter-Threading-Model">
+ *   <title>Threading Model</title>
+ *   <para>Clutter is <emphasis>thread-aware</emphasis>: all operations
+ *   performed by Clutter are assumed to be under the big Clutter lock,
+ *   which is created when the threading is initialized through
+ *   clutter_threads_init().</para>
+ *   <example id="example-Thread-Init">
+ *     <title>Thread Initialization</title>
+ *     <para>The code below shows how to correctly initialize Clutter
+ *     in a multi-threaded environment. These operations are mandatory for
+ *     applications that wish to use threads with Clutter.</para>
+ *     <programlisting>
+ * int
+ * main (int argc, char *argv[])
+ * {
+ *   /&ast; initialize GLib's threading support &ast;/
+ *   g_thread_init (NULL);
+ *
+ *   /&ast; initialize Clutter's threading support &ast;/
+ *   clutter_threads_init ();
+ *
+ *   /&ast; initialize Clutter &ast;/
+ *   clutter_init (&amp;argc, &amp;argv);
+ *
+ *   /&ast; program code &ast;/
+ *
+ *   /&ast; acquire the main lock &ast;/
+ *   clutter_threads_enter ();
+ *
+ *   /&ast; start the main loop &ast;/
+ *   clutter_main ();
+ *
+ *   /&ast; release the main lock &ast;/
+ *   clutter_threads_leave ();
+ *
+ *   /&ast; clean up &ast;/
+ *   return 0;
+ * }
+ *     </programlisting>
+ *   </example>
+ *   <para>This threading model has the caveat that it is only safe to call
+ *   Clutter's API when the lock has been acquired &mdash; which happens
+ *   between pairs of clutter_threads_enter() and clutter_threads_leave()
+ *   calls.</para>
+ *   <para>The only safe and portable way to use the Clutter API in a
+ *   multi-threaded environment is to never access the API from a thread that
+ *   did not call clutter_init() and clutter_main().</para>
+ *   <para>The common pattern for using threads with Clutter is to use worker
+ *   threads to perform blocking operations and then install idle or timeour
+ *   sources with the result when the thread finished.</para>
+ *   <para>Clutter provides thread-aware variants of g_idle_add() and
+ *   g_timeout_add() that acquire the Clutter lock before invoking the provided
+ *   callback: clutter_threads_add_idle() and
+ *   clutter_threads_add_timeout().</para>
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,6 +94,10 @@
 #include <glib/gi18n-lib.h>
 #include <locale.h>
 
+#ifdef USE_GDKPIXBUF
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#endif
+
 #include "clutter-event.h"
 #include "clutter-backend.h"
 #include "clutter-main.h"
@@ -70,13 +130,17 @@ static gboolean clutter_use_fuzzy_picking    = FALSE;
 
 static guint clutter_default_fps             = 60;
 
-static PangoDirection clutter_text_direction = PANGO_DIRECTION_LTR;
+static PangoDirection clutter_text_direction = CLUTTER_TEXT_DIRECTION_LTR;
 
 static guint clutter_main_loop_level         = 0;
 static GSList *main_loops                    = NULL;
 
 guint clutter_debug_flags = 0;  /* global clutter debug flag */
 
+const guint clutter_major_version = CLUTTER_MAJOR_VERSION;
+const guint clutter_minor_version = CLUTTER_MINOR_VERSION;
+const guint clutter_micro_version = CLUTTER_MICRO_VERSION;
+
 #ifdef CLUTTER_ENABLE_DEBUG
 static const GDebugKey clutter_debug_keys[] = {
   { "misc", CLUTTER_DEBUG_MISC },
@@ -95,7 +159,8 @@ static const GDebugKey clutter_debug_keys[] = {
   { "multistage", CLUTTER_DEBUG_MULTISTAGE },
   { "animation", CLUTTER_DEBUG_ANIMATION },
   { "layout", CLUTTER_DEBUG_LAYOUT },
-  { "nop-picking", CLUTTER_DEBUG_NOP_PICKING }
+  { "nop-picking", CLUTTER_DEBUG_NOP_PICKING },
+  { "dump-pick-buffers", CLUTTER_DEBUG_DUMP_PICK_BUFFERS }
 };
 #endif /* CLUTTER_ENABLE_DEBUG */
 
@@ -289,19 +354,6 @@ clutter_get_motion_events_enabled (void)
 
 guint _clutter_pix_to_id (guchar pixel[4]);
 
-static inline void init_bits (void)
-{
-  ClutterMainContext *ctx;
-
-  static gboolean done = FALSE;
-  if (G_LIKELY (done))
-    return;
-
-  ctx = _clutter_context_get_default ();
-
-  done = TRUE;
-}
-
 void
 _clutter_id_to_color (guint id, ClutterColor *col)
 {
@@ -312,9 +364,11 @@ _clutter_id_to_color (guint id, ClutterColor *col)
 
   /* compute the numbers we'll store in the components */
   red   = (id >> (ctx->fb_g_mask_used+ctx->fb_b_mask_used))
-                & (0xff >> (8-ctx->fb_r_mask_used));
-  green = (id >> ctx->fb_b_mask_used) & (0xff >> (8-ctx->fb_g_mask_used));
-  blue  = (id)  & (0xff >> (8-ctx->fb_b_mask_used));
+        & (0xff >> (8-ctx->fb_r_mask_used));
+  green = (id >> ctx->fb_b_mask_used)
+        & (0xff >> (8-ctx->fb_g_mask_used));
+  blue  = (id)
+        & (0xff >> (8-ctx->fb_b_mask_used));
 
   /* shift left bits a bit and add one, this circumvents
    * at least some potential rounding errors in GL/GLES
@@ -336,6 +390,18 @@ _clutter_id_to_color (guint id, ClutterColor *col)
   col->green = green;
   col->blue  = blue;
   col->alpha = 0xff;
+
+  /* XXX: We rotate the nibbles of the colors here so that there is a
+   * visible variation between colors of sequential actor identifiers;
+   * otherwise pick buffers dumped to an image will pretty much just look
+   * black.
+   */
+  if (G_UNLIKELY (clutter_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS))
+    {
+      col->red   = (col->red << 4)   | (col->red >> 4);
+      col->green = (col->green << 4) | (col->green >> 4);
+      col->blue  = (col->blue << 4)  | (col->blue >> 4);
+    }
 }
 
 guint
@@ -350,9 +416,28 @@ _clutter_pixel_to_id (guchar pixel[4])
   /* reduce the pixel components to the number of bits actually used of the
    * 8bits.
    */
-  red   = pixel[0] >> (8 - ctx->fb_r_mask);
-  green = pixel[1] >> (8 - ctx->fb_g_mask);
-  blue  = pixel[2] >> (8 - ctx->fb_b_mask);
+  if (G_UNLIKELY (clutter_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS))
+    {
+      guchar tmp;
+
+      /* XXX: In _clutter_id_to_color we rotated the nibbles of the colors so
+       * that there is a visible variation between colors of sequential actor
+       * identifiers (otherwise pick buffers dumped to an image will pretty
+       * much just look black.) Here we reverse that rotation.
+       */
+      tmp = ((pixel[0] << 4) | (pixel[0] >> 4));
+      red = tmp >> (8 - ctx->fb_r_mask);
+      tmp = ((pixel[1] << 4) | (pixel[1] >> 4));
+      green = tmp >> (8 - ctx->fb_g_mask);
+      tmp = ((pixel[2] << 4) | (pixel[2] >> 4));
+      blue = tmp >> (8 - ctx->fb_b_mask);
+    }
+  else
+    {
+      red   = pixel[0] >> (8 - ctx->fb_r_mask);
+      green = pixel[1] >> (8 - ctx->fb_g_mask);
+      blue  = pixel[2] >> (8 - ctx->fb_b_mask);
+    }
 
   /* divide potentially by two if 'fuzzy' */
   red   = red   >> (ctx->fb_r_mask - ctx->fb_r_mask_used);
@@ -360,12 +445,79 @@ _clutter_pixel_to_id (guchar pixel[4])
   blue  = blue  >> (ctx->fb_b_mask - ctx->fb_b_mask_used);
 
   /* combine the correct per component values into the final id */
-  id =  blue + (green <<  ctx->fb_b_mask_used)
-          + (red << (ctx->fb_b_mask_used + ctx->fb_g_mask_used));
+  id = blue
+     + (green <<  ctx->fb_b_mask_used)
+     + (red << (ctx->fb_b_mask_used + ctx->fb_g_mask_used));
 
   return id;
 }
 
+#ifdef USE_GDKPIXBUF
+static void
+pixbuf_free (guchar *pixels, gpointer data)
+{
+  g_free (pixels);
+}
+#endif
+
+static void
+read_pixels_to_file (char *filename_stem,
+                     int x,
+                     int y,
+                     int width,
+                     int height)
+{
+#ifdef USE_GDKPIXBUF
+  GLubyte *data;
+  GdkPixbuf *pixbuf;
+  static int read_count = 0;
+
+  data = g_malloc (4 * width * height);
+  cogl_read_pixels (x, y, width, height,
+                    COGL_READ_PIXELS_COLOR_BUFFER,
+                    COGL_PIXEL_FORMAT_RGBA_8888,
+                    data);
+  pixbuf = gdk_pixbuf_new_from_data (data,
+                                     GDK_COLORSPACE_RGB,
+                                     TRUE, /* has alpha */
+                                     8, /* bits per sample */
+                                     width, /* width */
+                                     height, /* height */
+                                     width * 4, /* rowstride */
+                                     pixbuf_free, /* callback to free data */
+                                     NULL); /* callback data */
+  if (pixbuf)
+    {
+      char *filename = g_strdup_printf ("%s-%05d.png",
+                                        filename_stem,
+                                        read_count);
+      GError *error = NULL;
+
+      if (!gdk_pixbuf_save (pixbuf, filename, "png", &error, NULL))
+        {
+          g_warning ("Failed to save pick buffer to file %s: %s",
+                     filename, error->message);
+          g_error_free (error);
+        }
+
+      g_free (filename);
+      g_object_unref (pixbuf);
+      read_count++;
+    }
+#else /* !USE_GDKPIXBUF */
+  {
+    static gboolean seen = FALSE;
+
+    if (!seen)
+      {
+        g_warning ("dumping buffers to an image isn't supported on platforms "
+                   "without gdk pixbuf support\n");
+        seen = TRUE;
+      }
+  }
+#endif /* USE_GDKPIXBUF */
+}
+
 ClutterActor *
 _clutter_do_pick (ClutterStage   *stage,
 		  gint            x,
@@ -374,8 +526,7 @@ _clutter_do_pick (ClutterStage   *stage,
 {
   ClutterMainContext *context;
   guchar              pixel[4] = { 0xff, 0xff, 0xff, 0xff };
-  GLint               viewport[4];
-  CoglColor           white;
+  CoglColor           stage_pick_id;
   guint32             id;
   GLboolean           dither_was_on;
 
@@ -389,10 +540,12 @@ _clutter_do_pick (ClutterStage   *stage,
   /* needed for when a context switch happens */
   _clutter_stage_maybe_setup_viewport (stage);
 
-  cogl_clip_push_window_rect (x, y, 1, 1);
-  cogl_color_set_from_4ub (&white, 255, 255, 255, 255);
+  if (G_LIKELY (!(clutter_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
+    cogl_clip_push_window_rectangle (x, y, 1, 1);
+
   cogl_disable_fog ();
-  cogl_clear (&white,
+  cogl_color_set_from_4ub (&stage_pick_id, 255, 255, 255, 255);
+  cogl_clear (&stage_pick_id,
 	      COGL_BUFFER_BIT_COLOR |
 	      COGL_BUFFER_BIT_DEPTH);
 
@@ -407,16 +560,25 @@ _clutter_do_pick (ClutterStage   *stage,
   context->pick_mode = mode;
   clutter_actor_paint (CLUTTER_ACTOR (stage));
   context->pick_mode = CLUTTER_PICK_NONE;
-  cogl_clip_pop ();
 
-  /* Calls should work under both GL and GLES, note GLES needs RGBA */
-  glGetIntegerv(GL_VIEWPORT, viewport);
+  if (G_LIKELY (!(clutter_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
+    cogl_clip_pop ();
 
   /* Make sure Cogl flushes any batched geometry to the GPU driver */
   cogl_flush ();
 
   /* Read the color of the screen co-ords pixel */
-  glReadPixels (x, viewport[3] - y -1, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel);
+  cogl_read_pixels (x, y, 1, 1,
+                    COGL_READ_PIXELS_COLOR_BUFFER,
+                    COGL_PIXEL_FORMAT_RGBA_8888,
+                    pixel);
+
+  if (G_UNLIKELY (clutter_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS))
+    {
+      read_pixels_to_file ("pick-buffer", 0, 0,
+                           clutter_actor_get_width (CLUTTER_ACTOR (stage)),
+                           clutter_actor_get_height (CLUTTER_ACTOR (stage)));
+    }
 
   /* Restore whether GL_DITHER was enabled */
   if (dither_was_on)
@@ -430,7 +592,7 @@ _clutter_do_pick (ClutterStage   *stage,
   return clutter_get_actor_by_gid (id);
 }
 
-static PangoDirection
+static ClutterTextDirection
 clutter_get_text_direction (void)
 {
   PangoDirection dir = PANGO_DIRECTION_LTR;
@@ -440,9 +602,9 @@ clutter_get_text_direction (void)
   if (direction && *direction != '\0')
     {
       if (strcmp (direction, "rtl") == 0)
-        dir = PANGO_DIRECTION_RTL;
+        dir = CLUTTER_TEXT_DIRECTION_RTL;
       else if (strcmp (direction, "ltr") == 0)
-        dir = PANGO_DIRECTION_LTR;
+        dir = CLUTTER_TEXT_DIRECTION_LTR;
     }
   else
     {
@@ -455,9 +617,9 @@ clutter_get_text_direction (void)
       char *e = _("default:LTR");
 
       if (strcmp (e, "default:RTL") == 0)
-        dir = PANGO_DIRECTION_RTL;
+        dir = CLUTTER_TEXT_DIRECTION_RTL;
       else if (strcmp (e, "default:LTR") == 0)
-        dir = PANGO_DIRECTION_LTR;
+        dir = CLUTTER_TEXT_DIRECTION_LTR;
       else
         g_warning ("Whoever translated default:LTR did so wrongly.");
     }
@@ -472,10 +634,16 @@ update_pango_context (ClutterBackend *backend,
   PangoFontDescription *font_desc;
   const cairo_font_options_t *font_options;
   const gchar *font_name;
+  PangoDirection pango_dir;
   gdouble resolution;
 
   /* update the text direction */
-  pango_context_set_base_dir (context, clutter_text_direction);
+  if (clutter_text_direction == CLUTTER_TEXT_DIRECTION_RTL)
+    pango_dir = PANGO_DIRECTION_RTL;
+  else
+    pango_dir = PANGO_DIRECTION_LTR;
+
+  pango_context_set_base_dir (context, pango_dir);
 
   /* get the configuration for the PangoContext from the backend */
   font_name = clutter_backend_get_font_name (backend);
@@ -524,6 +692,7 @@ _clutter_context_create_pango_context (ClutterMainContext *self)
 
   context = cogl_pango_font_map_create_context (self->font_map);
   update_pango_context (self->backend, context);
+  pango_context_set_language (context, pango_language_get_default ());
 
   return context;
 }
@@ -607,14 +776,14 @@ clutter_main (void)
 static void
 clutter_threads_impl_lock (void)
 {
-  if (clutter_threads_mutex)
+  if (G_LIKELY (clutter_threads_mutex != NULL))
     g_mutex_lock (clutter_threads_mutex);
 }
 
 static void
 clutter_threads_impl_unlock (void)
 {
-  if (clutter_threads_mutex)
+  if (G_LIKELY (clutter_threads_mutex != NULL))
     g_mutex_unlock (clutter_threads_mutex);
 }
 
@@ -629,6 +798,8 @@ clutter_threads_impl_unlock (void)
  *
  * This function must be called before clutter_init().
  *
+ * It is safe to call this function multiple times.
+ *
  * Since: 0.4
  */
 void
@@ -637,6 +808,9 @@ clutter_threads_init (void)
   if (!g_thread_supported ())
     g_error ("g_thread_init() must be called before clutter_threads_init()");
 
+  if (clutter_threads_mutex != NULL)
+    return;
+
   clutter_threads_mutex = g_mutex_new ();
 
   if (!clutter_threads_lock)
@@ -1080,6 +1254,7 @@ _clutter_context_get_default (void)
 
       ClutterCntx = ctx = g_new0 (ClutterMainContext, 1);
 
+      /* create the default backend */
       ctx->backend = g_object_new (_clutter_backend_impl_get_type (), NULL);
 
       ctx->is_initialized = FALSE;
@@ -1126,8 +1301,8 @@ clutter_arg_direction_cb (const char *key,
                           gpointer    user_data)
 {
   clutter_text_direction =
-    (strcmp (value, "rtl") == 0) ? PANGO_DIRECTION_RTL
-                                 : PANGO_DIRECTION_LTR;
+    (strcmp (value, "rtl") == 0) ? CLUTTER_TEXT_DIRECTION_RTL
+                                 : CLUTTER_TEXT_DIRECTION_LTR;
 
   return TRUE;
 }
@@ -1227,6 +1402,8 @@ clutter_init_real (GError **error)
   /* Now we can safely assume we have a valid GL context and can
    * start issueing cogl commands
   */
+  /* - will call to backend and cogl */
+  _clutter_feature_init ();
 
   /*
    * Resolution requires display to be open, so can only be queried after
@@ -1239,12 +1416,11 @@ clutter_init_real (GError **error)
   resolution = clutter_backend_get_resolution (ctx->backend);
   cogl_pango_font_map_set_resolution (ctx->font_map, resolution);
 
-  if (G_LIKELY (!clutter_disable_mipmap_text))
+  if (!clutter_disable_mipmap_text)
     cogl_pango_font_map_set_use_mipmapping (ctx->font_map, TRUE);
 
   clutter_text_direction = clutter_get_text_direction ();
 
-
   /* Figure out framebuffer masks used for pick */
   cogl_get_bitmasks (&ctx->fb_r_mask, &ctx->fb_g_mask, &ctx->fb_b_mask, NULL);
 
@@ -1263,9 +1439,6 @@ clutter_init_real (GError **error)
   /* Initiate event collection */
   _clutter_backend_init_events (ctx->backend);
 
-  /* finally features - will call to backend and cogl */
-  _clutter_feature_init ();
-
   clutter_is_initialized = TRUE;
   ctx->is_initialized = TRUE;
 
@@ -1552,8 +1725,10 @@ clutter_init_with_args (int            *argc,
 
   if (!ctx->defer_display_setup)
     {
+#if 0
       if (argc && *argc > 0 && *argv)
 	g_set_prgname ((*argv)[0]);
+#endif
 
       context = g_option_context_new (parameter_string);
 
@@ -1658,8 +1833,10 @@ clutter_init (int    *argc,
 
   if (!ctx->defer_display_setup)
     {
+#if 0
       if (argc && *argc > 0 && *argv)
 	g_set_prgname ((*argv)[0]);
+#endif
 
       /* parse_args will trigger backend creation and things like
        * DISPLAY connection etc.
@@ -1995,36 +2172,13 @@ clutter_do_event (ClutterEvent *event)
   _clutter_stage_queue_event (event->any.stage, event);
 }
 
-/**
- * _clutter_process_event
- * @event: a #ClutterEvent.
- *
- * Does the actual work of processing an event that was queued earlier
- * out of clutter_do_event().
- */
-void
-_clutter_process_event (ClutterEvent *event)
+static void
+_clutter_process_event_details (ClutterActor        *stage,
+                                ClutterMainContext  *context,
+                                ClutterEvent        *event)
 {
-  /* FIXME: This should probably be clutter_cook_event() - it would
-   * take a raw event from the backend and 'cook' it so its more tasty.
-   *
-  */
-  ClutterMainContext  *context;
-  ClutterBackend      *backend;
-  ClutterActor        *stage;
   ClutterInputDevice  *device = NULL;
 
-  context = _clutter_context_get_default ();
-  backend = context->backend;
-  stage   = CLUTTER_ACTOR(event->any.stage);
-
-  if (!stage)
-    return;
-
-  CLUTTER_TIMESTAMP (EVENT, "Event received");
-
-  context->last_event_time = clutter_event_get_time (event);
-
   switch (event->type)
     {
       case CLUTTER_NOTHING:
@@ -2222,6 +2376,37 @@ _clutter_process_event (ClutterEvent *event)
     }
 }
 
+/**
+ * _clutter_process_event
+ * @event: a #ClutterEvent.
+ *
+ * Does the actual work of processing an event that was queued earlier
+ * out of clutter_do_event().
+ */
+void
+_clutter_process_event (ClutterEvent *event)
+{
+  ClutterMainContext  *context;
+  ClutterBackend      *backend;
+  ClutterActor        *stage;
+
+  context = _clutter_context_get_default ();
+  backend = context->backend;
+  stage   = CLUTTER_ACTOR(event->any.stage);
+
+  if (!stage)
+    return;
+
+  CLUTTER_TIMESTAMP (EVENT, "Event received");
+
+  context->last_event_time = clutter_event_get_time (event);
+
+  context->current_event = event;
+  _clutter_process_event_details (stage, context, event);
+  context->current_event = NULL;
+}
+
+
 /**
  * clutter_get_actor_by_gid
  * @id: a #ClutterActor ID.
@@ -2888,3 +3073,53 @@ _clutter_run_repaint_functions (void)
   if (reinvoke_list)
     context->repaint_funcs = reinvoke_list;
 }
+
+/**
+ * clutter_check_version:
+ * @major: major version, like 1 in 1.2.3
+ * @minor: minor version, like 2 in 1.2.3
+ * @micro: micro version, like 3 in 1.2.3
+ *
+ * Run-time version check, to check the version the Clutter library
+ * that an application is currently linked against
+ *
+ * This is the run-time equivalent of the compile-time %CLUTTER_CHECK_VERSION
+ * pre-processor macro
+ *
+ * Return value: %TRUE if the version of the Clutter library is
+ *   greater than (@major, @minor, @micro), and %FALSE otherwise
+ *
+ * Since: 1.2
+ */
+gboolean
+clutter_check_version (guint major,
+                       guint minor,
+                       guint micro)
+{
+  return (clutter_major_version > major ||
+          (clutter_major_version == major &&
+           clutter_minor_version > minor) ||
+          (clutter_major_version == major &&
+           clutter_minor_version == minor &&
+           clutter_micro_version >= micro));
+}
+
+/**
+ * clutter_get_default_text_direction:
+ *
+ * Retrieves the default direction for the text. The text direction is
+ * determined by the locale and/or by the %CLUTTER_TEXT_DIRECTION environment
+ * variable
+ *
+ * The default text direction can be overridden on a per-actor basis by using
+ * clutter_actor_set_text_direction()
+ *
+ * Return value: the default text direction
+ *
+ * Since: 1.2
+ */
+ClutterTextDirection
+clutter_get_default_text_direction (void)
+{
+  return clutter_text_direction;
+}
