@@ -39,25 +39,13 @@ G_BEGIN_DECLS
 #define CLUTTER_TYPE_FOG                (clutter_fog_get_type ())
 #define CLUTTER_TYPE_STAGE              (clutter_stage_get_type())
 
-#define CLUTTER_STAGE(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
-  CLUTTER_TYPE_STAGE, ClutterStage))
+#define CLUTTER_STAGE(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), CLUTTER_TYPE_STAGE, ClutterStage))
+#define CLUTTER_STAGE_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), CLUTTER_TYPE_STAGE, ClutterStageClass))
+#define CLUTTER_IS_STAGE(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CLUTTER_TYPE_STAGE))
+#define CLUTTER_IS_STAGE_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), CLUTTER_TYPE_STAGE))
+#define CLUTTER_STAGE_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), CLUTTER_TYPE_STAGE, ClutterStageClass))
 
-#define CLUTTER_STAGE_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST ((klass), \
-  CLUTTER_TYPE_STAGE, ClutterStageClass))
-
-#define CLUTTER_IS_STAGE(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
-  CLUTTER_TYPE_STAGE))
-
-#define CLUTTER_IS_STAGE_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_TYPE ((klass), \
-  CLUTTER_TYPE_STAGE))
-
-#define CLUTTER_STAGE_GET_CLASS(obj) \
-  (G_TYPE_INSTANCE_GET_CLASS ((obj), \
-  CLUTTER_TYPE_STAGE, ClutterStageClass))
+#ifndef CLUTTER_DISABLE_DEPRECATED
 
 /**
  * CLUTTER_STAGE_WIDTH:
@@ -65,9 +53,10 @@ G_BEGIN_DECLS
  * Macro that evaluates to the width of the default stage
  *
  * Since: 0.2
+ *
+ * Deprecated: 1.2: Use clutter_actor_get_width() instead
  */
-#define CLUTTER_STAGE_WIDTH() \
- (clutter_actor_get_width (clutter_stage_get_default ()))
+#define CLUTTER_STAGE_WIDTH()           (clutter_actor_get_width (clutter_stage_get_default ()))
 
 /**
  * CLUTTER_STAGE_HEIGHT:
@@ -75,9 +64,12 @@ G_BEGIN_DECLS
  * Macro that evaluates to the height of the default stage
  *
  * Since: 0.2
+ *
+ * Deprecated: 1.2: use clutter_actor_get_height() instead
  */
-#define CLUTTER_STAGE_HEIGHT() \
- (clutter_actor_get_height (clutter_stage_get_default ()))
+#define CLUTTER_STAGE_HEIGHT()          (clutter_actor_get_height (clutter_stage_get_default ()))
+
+#endif /* !CLUTTER_DISABLE_DEPRECATED */
 
 /**
  * ClutterPickMode:
@@ -237,15 +229,21 @@ void                  clutter_stage_get_fog            (ClutterStage *stage,
 void                  clutter_stage_set_key_focus      (ClutterStage *stage,
                                                         ClutterActor *actor);
 ClutterActor *        clutter_stage_get_key_focus      (ClutterStage *stage);
-
-/* New experiental calls */
 void                  clutter_stage_ensure_current     (ClutterStage *stage);
 void                  clutter_stage_queue_redraw       (ClutterStage *stage);
 gboolean              clutter_stage_is_default         (ClutterStage *stage);
 void                  clutter_stage_ensure_viewport    (ClutterStage *stage);
 void                  clutter_stage_ensure_redraw      (ClutterStage *stage);
 
-/* Commodity macro */
+void     clutter_stage_set_throttle_motion_events (ClutterStage *stage,
+                                                   gboolean      throttle);
+gboolean clutter_stage_get_throttle_motion_events (ClutterStage *stage);
+
+void                  clutter_stage_set_use_alpha      (ClutterStage *stage,
+                                                        gboolean      use_alpha);
+gboolean              clutter_stage_get_use_alpha      (ClutterStage *stage);
+
+/* Commodity macro, for mallum only */
 #define clutter_stage_add(stage,actor)                  G_STMT_START {  \
   if (CLUTTER_IS_STAGE ((stage)) && CLUTTER_IS_ACTOR ((actor)))         \
     {                                                                   \
