@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
- *
- *
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
@@ -34,6 +34,12 @@
  * clutter_texture_set_from_file() functions are used to copy image
  * data into texture memory and subsequently realize the texture.
  *
+ * If texture reads are supported by underlying GL implementation,
+ * unrealizing frees image data from texture memory moving to main
+ * system memory. Re-realizing then performs the opposite operation.
+ * This process allows basic management of commonly limited available
+ * texture memory.
+ *
  * Note: a ClutterTexture will scale its contents to fit the bounding
  * box requested using clutter_actor_set_size(). To display an area of
  * a texture without scaling, you should set the clip area using
@@ -80,9 +86,16 @@ struct _ClutterTexturePrivate
   ClutterActor                *fbo_source;
   CoglHandle                   fbo_handle;
 
+  /* Non video memory copy of image data */
+  guint                        local_data_width, local_data_height;
+  guint                        local_data_rowstride;
+  guint                        local_data_has_alpha;
+  guchar                      *local_data;
+
   guint                        sync_actor_size : 1;
   guint                        repeat_x : 1;
   guint                        repeat_y : 1;
+  guint                        in_dispose : 1;
   guint                        keep_aspect_ratio : 1;
   guint                        load_size_async : 1;
   guint                        load_data_async : 1;
@@ -159,6 +172,12 @@ static GStaticMutex upload_list_mutex = G_STATIC_MUTEX_INIT;
 static void
 texture_fbo_free_resources (ClutterTexture *texture);
 
+static void
+clutter_texture_save_to_local_data (ClutterTexture *texture);
+
+static void
+clutter_texture_load_from_local_data (ClutterTexture *texture);
+
 GQuark
 clutter_texture_error_quark (void)
 {
@@ -222,6 +241,14 @@ clutter_texture_unrealize (ClutterActor *actor)
   if (priv->material == COGL_INVALID_HANDLE)
     return;
 
+  /* there's no need to read the pixels back when unrealizing inside
+   * a dispose run, and the dispose() call will release the GL
+   * texture data as well, so we can safely bail out now
+   */
+  if ((CLUTTER_PRIVATE_FLAGS (actor) & CLUTTER_ACTOR_IN_DESTRUCTION) ||
+      priv->in_dispose)
+    return;
+
   CLUTTER_MARK();
 
   if (priv->fbo_source != NULL)
@@ -233,6 +260,24 @@ clutter_texture_unrealize (ClutterActor *actor)
       return;
     }
 
+  if (clutter_feature_available (CLUTTER_FEATURE_TEXTURE_READ_PIXELS))
+    {
+      /* Move image data from video to main memory.
+       * GL/ES cant do this - it probably makes sense
+       * to move this kind of thing into a ClutterProxyTexture
+       * where this behaviour can be better controlled.
+       *
+       * Or make it controllable via a property.
+       */
+      if (priv->local_data == NULL)
+	{
+	  clutter_texture_save_to_local_data (texture);
+	  CLUTTER_NOTE (TEXTURE, "moved pixels into system mem");
+	}
+
+      texture_free_gl_resources (texture);
+    }
+
   CLUTTER_NOTE (TEXTURE, "Texture unrealized");
 }
 
@@ -282,14 +327,25 @@ clutter_texture_realize (ClutterActor *actor)
       return;
     }
 
-  /* If the texture is not a FBO, then realization is a no-op but
-   * we still want to be in REALIZED state to maintain invariants.
-   * We may have already created the texture if someone set some
-   * data earlier, or we may create it later if someone sets some
-   * data later. The fact that we may have created it earlier is
-   * really a bug, since it means ClutterTexture can have GL
-   * resources without being realized.
-   */
+  if (priv->local_data != NULL)
+    {
+      /* Move any local image data we have from unrealization
+       * back into video memory.
+      */
+      clutter_texture_load_from_local_data (texture);
+    }
+  else
+    {
+      /* If we have no data, then realization is a no-op but
+       * we still want to be in REALIZED state to maintain
+       * invariants. We may have already created the texture
+       * if someone set some data earlier, or we may create it
+       * later if someone sets some data later. The fact that
+       * we may have created it earlier is really a bug, since
+       * it means ClutterTexture can have GL resources without
+       * being realized.
+       */
+    }
 
   CLUTTER_NOTE (TEXTURE, "Texture realized");
 }
@@ -645,10 +701,27 @@ static void
 clutter_texture_dispose (GObject *object)
 {
   ClutterTexture *texture = CLUTTER_TEXTURE (object);
+  ClutterTexturePrivate *priv;
+
+  priv = texture->priv;
+
+  /* mark that we are in dispose, so when the parent class'
+   * dispose implementation will call unrealize on us we'll
+   * not try to copy back the resources from video memory
+   * to system memory
+   */
+  if (!priv->in_dispose)
+    priv->in_dispose = TRUE;
 
   texture_free_gl_resources (texture);
   texture_fbo_free_resources (texture);
 
+  if (priv->local_data != NULL)
+    {
+      g_free (priv->local_data);
+      priv->local_data = NULL;
+    }
+
   clutter_texture_async_load_cancel (texture);
 
   G_OBJECT_CLASS (clutter_texture_parent_class)->dispose (object);
@@ -1104,9 +1177,78 @@ clutter_texture_init (ClutterTexture *self)
   priv->sync_actor_size   = TRUE;
   priv->material          = cogl_material_new ();
   priv->fbo_handle        = COGL_INVALID_HANDLE;
+  priv->local_data        = NULL;
   priv->keep_aspect_ratio = FALSE;
 }
 
+static void
+clutter_texture_save_to_local_data (ClutterTexture *texture)
+{
+  ClutterTexturePrivate *priv;
+  int                    bpp;
+  CoglPixelFormat        pixel_format;
+  CoglHandle             cogl_texture;
+
+  priv = texture->priv;
+
+  if (priv->local_data)
+    {
+      g_free (priv->local_data);
+      priv->local_data = NULL;
+    }
+
+  if (priv->material == COGL_INVALID_HANDLE)
+    return;
+
+  cogl_texture = clutter_texture_get_cogl_texture (texture);
+
+  priv->local_data_width = cogl_texture_get_width (cogl_texture);
+  priv->local_data_height = cogl_texture_get_height (cogl_texture);
+  pixel_format = cogl_texture_get_format (cogl_texture);
+  priv->local_data_has_alpha = pixel_format & COGL_A_BIT;
+  bpp = priv->local_data_has_alpha ? 4 : 3;
+
+  /* Align to 4 bytes */
+  priv->local_data_rowstride = (priv->local_data_width * bpp + 3) & ~3;
+
+  priv->local_data = g_malloc (priv->local_data_rowstride
+			       * priv->local_data_height);
+
+  if (cogl_texture_get_data (cogl_texture,
+			     priv->local_data_has_alpha
+			     ? COGL_PIXEL_FORMAT_RGBA_8888_PRE
+			     : COGL_PIXEL_FORMAT_RGB_888,
+			     priv->local_data_rowstride,
+			     priv->local_data) == 0)
+    {
+      g_free (priv->local_data);
+      priv->local_data = NULL;
+    }
+}
+
+static void
+clutter_texture_load_from_local_data (ClutterTexture *texture)
+{
+  ClutterTexturePrivate *priv;
+
+  priv = texture->priv;
+
+  if (priv->local_data == NULL)
+    return;
+
+  clutter_texture_set_from_rgb_data (texture,
+				     priv->local_data,
+				     priv->local_data_has_alpha,
+				     priv->local_data_width,
+				     priv->local_data_height,
+				     priv->local_data_rowstride,
+				     priv->local_data_has_alpha ? 4: 3,
+				     CLUTTER_TEXTURE_RGB_FLAG_PREMULT, NULL);
+
+  g_free (priv->local_data);
+  priv->local_data = NULL;
+}
+
 /**
  * clutter_texture_get_cogl_material:
  * @texture: A #ClutterTexture
@@ -1251,7 +1393,15 @@ clutter_texture_set_cogl_texture (ClutterTexture  *texture,
   /* Remove old texture */
   texture_free_gl_resources (texture);
 
+  /* Free any saved data so realization doesn't resend it to GL */
+  if (priv->local_data)
+    {
+      g_free (priv->local_data);
+      priv->local_data = NULL;
+    }
+
   /* Use the new texture */
+
   cogl_material_set_layer (priv->material, 0, cogl_tex);
 
   /* The material now holds a reference to the texture so we can
@@ -1282,7 +1432,8 @@ clutter_texture_set_cogl_texture (ClutterTexture  *texture,
   g_object_notify (G_OBJECT (texture), "cogl-texture");
 
   /* If resized actor may need resizing but paint() will do this */
-  clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
+  if (CLUTTER_ACTOR_IS_VISIBLE (texture))
+    clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
 }
 
 static gboolean
@@ -1369,10 +1520,13 @@ clutter_texture_set_from_rgb_data (ClutterTexture       *texture,
 				   ClutterTextureFlags   flags,
 				   GError              **error)
 {
+  ClutterTexturePrivate *priv;
   CoglPixelFormat source_format;
 
   g_return_val_if_fail (CLUTTER_IS_TEXTURE (texture), FALSE);
 
+  priv = texture->priv;
+
   /* Convert the flags to a CoglPixelFormat */
   if (has_alpha)
     {
@@ -1441,6 +1595,8 @@ clutter_texture_set_from_yuv_data (ClutterTexture     *texture,
 				   ClutterTextureFlags flags,
 				   GError            **error)
 {
+  ClutterTexturePrivate *priv;
+
   g_return_val_if_fail (CLUTTER_IS_TEXTURE (texture), FALSE);
 
   if (!clutter_feature_available (CLUTTER_FEATURE_TEXTURE_YUV))
@@ -1451,6 +1607,8 @@ clutter_texture_set_from_yuv_data (ClutterTexture     *texture,
       return FALSE;
     }
 
+  priv = texture->priv;
+
   /* Convert the flags to a CoglPixelFormat */
   if ((flags & CLUTTER_TEXTURE_YUV_FLAG_YUV2))
     {
@@ -1871,7 +2029,8 @@ clutter_texture_set_filter_quality (ClutterTexture        *texture,
 
       g_object_notify (G_OBJECT (texture), "filter-quality");
 
-      clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
+      if (CLUTTER_ACTOR_IS_VISIBLE (texture))
+	clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
     }
 }
 
@@ -2044,8 +2203,11 @@ clutter_texture_set_area_from_rgb_data (ClutterTexture     *texture,
                                         ClutterTextureFlags flags,
                                         GError            **error)
 {
-  CoglPixelFormat source_format;
-  CoglHandle cogl_texture;
+  ClutterTexturePrivate *priv;
+  CoglPixelFormat        source_format;
+  CoglHandle             cogl_texture;
+
+  priv = texture->priv;
 
   if (has_alpha)
     {
@@ -2056,7 +2218,6 @@ clutter_texture_set_area_from_rgb_data (ClutterTexture     *texture,
 		       "Unsupported BPP");
 	  return FALSE;
 	}
-
       source_format = COGL_PIXEL_FORMAT_RGBA_8888;
     }
   else
@@ -2068,22 +2229,17 @@ clutter_texture_set_area_from_rgb_data (ClutterTexture     *texture,
 		       "Unsupported BPP");
 	  return FALSE;
 	}
-
       source_format = COGL_PIXEL_FORMAT_RGB_888;
     }
-
   if ((flags & CLUTTER_TEXTURE_RGB_FLAG_BGR))
     source_format |= COGL_BGR_BIT;
-
   if ((flags & CLUTTER_TEXTURE_RGB_FLAG_PREMULT))
     source_format |= COGL_PREMULT_BIT;
 
   /* attempt to realize ... */
   if (!CLUTTER_ACTOR_IS_REALIZED (texture) &&
       clutter_actor_get_stage (CLUTTER_ACTOR (texture)) != NULL)
-    {
-      clutter_actor_realize (CLUTTER_ACTOR (texture));
-    }
+    clutter_actor_realize (CLUTTER_ACTOR (texture));
 
   /* due to the fudging of clutter_texture_set_cogl_texture()
    * which allows setting a texture pre-realize, we may end
@@ -2115,7 +2271,8 @@ clutter_texture_set_area_from_rgb_data (ClutterTexture     *texture,
   /* rename signal */
   g_signal_emit (texture, texture_signals[PIXBUF_CHANGE], 0);
 
-  clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
+  if (CLUTTER_ACTOR_IS_VISIBLE (texture))
+    clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
 
   return TRUE;
 }
