@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
- *
- *
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #if !defined(__COGL_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
@@ -86,12 +86,12 @@ typedef enum {
  *
  * Return value: a handle to the new material
  */
-CoglHandle
-cogl_material_new (void);
+CoglHandle cogl_material_new (void);
+
+#ifndef COGL_DISABLE_DEPRECATED
 
 /**
  * cogl_material_copy:
- * @source: the handle for the material to copy
  *
  * Creates a new material handle with the configuration copied from the
  * source material.
@@ -106,10 +106,7 @@ cogl_material_new (void);
  *
  * Since: 1.2
  */
-CoglHandle
-cogl_material_copy (CoglHandle source);
-
-#ifndef COGL_DISABLE_DEPRECATED
+CoglHandle cogl_material_copy (CoglHandle source);
 
 /**
  * cogl_material_ref:
@@ -123,8 +120,7 @@ cogl_material_copy (CoglHandle source);
  *
  * Deprecated: 1.2: Use cogl_handle_ref() instead
  */
-CoglHandle
-cogl_material_ref (CoglHandle handle) G_GNUC_DEPRECATED;
+CoglHandle cogl_material_ref (CoglHandle handle);
 
 /**
  * cogl_material_unref:
@@ -136,8 +132,7 @@ cogl_material_ref (CoglHandle handle) G_GNUC_DEPRECATED;
  *
  * Deprecated: 1.2: Use cogl_handle_unref() instead
  */
-void
-cogl_material_unref (CoglHandle handle) G_GNUC_DEPRECATED;
+void cogl_material_unref (CoglHandle handle);
 
 #endif /* COGL_DISABLE_DEPRECATED */
 
@@ -150,8 +145,7 @@ cogl_material_unref (CoglHandle handle) G_GNUC_DEPRECATED;
  * Return value: %TRUE if the handle references a #CoglMaterial,
  *   %FALSE otherwise
  */
-gboolean
-cogl_is_material (CoglHandle handle);
+gboolean cogl_is_material (CoglHandle handle);
 
 /**
  * cogl_material_set_color:
@@ -169,9 +163,8 @@ cogl_is_material (CoglHandle handle);
  *
  * Since: 1.0
  */
-void
-cogl_material_set_color (CoglHandle       material,
-                         const CoglColor *color);
+void cogl_material_set_color (CoglHandle       material,
+                              const CoglColor *color);
 
 /**
  * cogl_material_set_color4ub:
@@ -187,12 +180,11 @@ cogl_material_set_color (CoglHandle       material,
  *
  * Since: 1.0
  */
-void
-cogl_material_set_color4ub (CoglHandle material,
-			    guint8     red,
-                            guint8     green,
-                            guint8     blue,
-                            guint8     alpha);
+void cogl_material_set_color4ub (CoglHandle material,
+			         guint8     red,
+                                 guint8     green,
+                                 guint8     blue,
+                                 guint8     alpha);
 
 /**
  * cogl_material_set_color4f:
@@ -208,12 +200,11 @@ cogl_material_set_color4ub (CoglHandle material,
  *
  * Since: 1.0
  */
-void
-cogl_material_set_color4f (CoglHandle material,
-                           float      red,
-                           float      green,
-                           float      blue,
-                           float      alpha);
+void cogl_material_set_color4f (CoglHandle material,
+                                float      red,
+                                float      green,
+                                float      blue,
+                                float      alpha);
 
 /**
  * cogl_material_get_color:
@@ -224,9 +215,8 @@ cogl_material_set_color4f (CoglHandle material,
  *
  * Since: 1.0
  */
-void
-cogl_material_get_color (CoglHandle  material,
-                         CoglColor  *color);
+void cogl_material_get_color (CoglHandle  material,
+                              CoglColor  *color);
 
 /**
  * cogl_material_set_ambient:
@@ -244,9 +234,8 @@ cogl_material_get_color (CoglHandle  material,
  *
  * Since: 1.0
  */
-void
-cogl_material_set_ambient (CoglHandle       material,
-			   const CoglColor *ambient);
+void cogl_material_set_ambient (CoglHandle       material,
+				const CoglColor *ambient);
 
 /**
  * cogl_material_get_ambient:
@@ -257,9 +246,8 @@ cogl_material_set_ambient (CoglHandle       material,
  *
  * Since: 1.0
  */
-void
-cogl_material_get_ambient (CoglHandle  material,
-                           CoglColor  *ambient);
+void cogl_material_get_ambient (CoglHandle  material,
+                                CoglColor  *ambient);
 
 /**
  * cogl_material_set_diffuse:
@@ -274,9 +262,8 @@ cogl_material_get_ambient (CoglHandle  material,
  *
  * Since: 1.0
  */
-void
-cogl_material_set_diffuse (CoglHandle       material,
-			   const CoglColor *diffuse);
+void cogl_material_set_diffuse (CoglHandle       material,
+				const CoglColor *diffuse);
 
 /**
  * cogl_material_get_diffuse:
@@ -287,9 +274,8 @@ cogl_material_set_diffuse (CoglHandle       material,
  *
  * Since: 1.0
  */
-void
-cogl_material_get_diffuse (CoglHandle  material,
-                           CoglColor  *diffuse);
+void cogl_material_get_diffuse (CoglHandle  material,
+                                CoglColor  *diffuse);
 
 /**
  * cogl_material_set_ambient_and_diffuse:
@@ -305,9 +291,8 @@ cogl_material_get_diffuse (CoglHandle  material,
  *
  * Since: 1.0
  */
-void
-cogl_material_set_ambient_and_diffuse (CoglHandle       material,
-				       const CoglColor *color);
+void cogl_material_set_ambient_and_diffuse (CoglHandle       material,
+					    const CoglColor *color);
 
 /**
  * cogl_material_set_specular:
@@ -322,9 +307,8 @@ cogl_material_set_ambient_and_diffuse (CoglHandle       material,
  *
  * Since: 1.0
  */
-void
-cogl_material_set_specular (CoglHandle       material,
-			    const CoglColor *specular);
+void cogl_material_set_specular (CoglHandle       material,
+				 const CoglColor *specular);
 
 /**
  * cogl_material_get_specular:
@@ -335,9 +319,8 @@ cogl_material_set_specular (CoglHandle       material,
  *
  * Since: 1.0
  */
-void
-cogl_material_get_specular (CoglHandle  material,
-                            CoglColor  *specular);
+void cogl_material_get_specular (CoglHandle  material,
+                                 CoglColor  *specular);
 
 /**
  * cogl_material_set_shininess:
@@ -352,10 +335,8 @@ cogl_material_get_specular (CoglHandle  material,
  *
  * Since: 1.0
  */
-void
-cogl_material_set_shininess (CoglHandle material,
-			     float      shininess);
-
+void cogl_material_set_shininess (CoglHandle material,
+				  float      shininess);
 /**
  * cogl_material_get_shininess:
  * @material: A CoglMaterial object
@@ -366,8 +347,7 @@ cogl_material_set_shininess (CoglHandle material,
  *
  * Since: 1.0
  */
-float
-cogl_material_get_shininess (CoglHandle material);
+float cogl_material_get_shininess (CoglHandle material);
 
 /**
  * cogl_material_set_emission:
@@ -382,9 +362,8 @@ cogl_material_get_shininess (CoglHandle material);
  *
  * Since: 1.0
  */
-void
-cogl_material_set_emission (CoglHandle       material,
-			    const CoglColor *emission);
+void cogl_material_set_emission (CoglHandle       material,
+				 const CoglColor *emission);
 
 /**
  * cogl_material_get_emission:
@@ -395,9 +374,8 @@ cogl_material_set_emission (CoglHandle       material,
  *
  * Since: 1.0
  */
-void
-cogl_material_get_emission (CoglHandle material,
-                            CoglColor *emission);
+void cogl_material_get_emission (CoglHandle material,
+                                 CoglColor *emission);
 
 /**
  * CoglMaterialAlphaFunc:
@@ -449,10 +427,9 @@ typedef enum {
  *
  * Since: 1.0
  */
-void
-cogl_material_set_alpha_test_function (CoglHandle            material,
-				       CoglMaterialAlphaFunc alpha_func,
-				       float                 alpha_reference);
+void cogl_material_set_alpha_test_function (CoglHandle            material,
+					    CoglMaterialAlphaFunc alpha_func,
+					    float                 alpha_reference);
 
 /**
  * cogl_material_set_blend:
@@ -538,10 +515,9 @@ cogl_material_set_alpha_test_function (CoglHandle            material,
  *
  * Since: 1.0
  */
-gboolean
-cogl_material_set_blend (CoglHandle  material,
-                         const char *blend_string,
-                         GError    **error);
+gboolean cogl_material_set_blend (CoglHandle  material,
+                                  const char *blend_string,
+                                  GError    **error);
 
 /**
  * cogl_material_set_blend_constant:
@@ -553,9 +529,8 @@ cogl_material_set_blend (CoglHandle  material,
  *
  * Since: 1.0
  */
-void
-cogl_material_set_blend_constant (CoglHandle material,
-                                  CoglColor *constant_color);
+void cogl_material_set_blend_constant (CoglHandle material,
+                                       CoglColor *constant_color);
 
 /**
  * cogl_material_set_layer:
@@ -576,10 +551,9 @@ cogl_material_set_blend_constant (CoglHandle material,
  *
  * Since: 1.0
  */
-void
-cogl_material_set_layer (CoglHandle material,
-			 int        layer_index,
-			 CoglHandle texture);
+void cogl_material_set_layer (CoglHandle material,
+			      int        layer_index,
+			      CoglHandle texture);
 
 /**
  * cogl_material_remove_layer:
@@ -588,9 +562,8 @@ cogl_material_set_layer (CoglHandle material,
  *
  * This function removes a layer from your material
  */
-void
-cogl_material_remove_layer (CoglHandle material,
-			    int        layer_index);
+void cogl_material_remove_layer (CoglHandle material,
+				 gint       layer_index);
 
 
 /**
@@ -687,7 +660,7 @@ cogl_material_remove_layer (CoglHandle material,
  */
 gboolean
 cogl_material_set_layer_combine (CoglHandle   material,
-				 int          layer_index,
+				 gint         layer_index,
 				 const char  *blend_string,
                                  GError     **error);
 
@@ -703,10 +676,9 @@ cogl_material_set_layer_combine (CoglHandle   material,
  *
  * Since: 1.0
  */
-void
-cogl_material_set_layer_combine_constant (CoglHandle  material,
-                                          int         layer_index,
-                                          CoglColor  *constant);
+void cogl_material_set_layer_combine_constant (CoglHandle  material,
+                                               int         layer_index,
+                                               CoglColor  *constant);
 
 /**
  * cogl_material_set_layer_matrix:
@@ -717,10 +689,9 @@ cogl_material_set_layer_combine_constant (CoglHandle  material,
  * This function lets you set a matrix that can be used to e.g. translate
  * and rotate a single layer of a material used to fill your geometry.
  */
-void
-cogl_material_set_layer_matrix (CoglHandle  material,
-				int         layer_index,
-				CoglMatrix *matrix);
+void cogl_material_set_layer_matrix (CoglHandle  material,
+				     int         layer_index,
+				     CoglMatrix *matrix);
 
 /**
  * cogl_material_get_layers:
@@ -734,8 +705,7 @@ cogl_material_set_layer_matrix (CoglHandle  material,
  *   functions. The list is owned by COGL and it  should not be modified or
  *   freed
  */
-G_CONST_RETURN GList *
-cogl_material_get_layers (CoglHandle material);
+G_CONST_RETURN GList *cogl_material_get_layers (CoglHandle material);
 
 /**
  * cogl_material_get_n_layers:
@@ -747,8 +717,7 @@ cogl_material_get_layers (CoglHandle material);
  *
  * Since: 1.0
  */
-int
-cogl_material_get_n_layers (CoglHandle material);
+int cogl_material_get_n_layers (CoglHandle material);
 
 /**
  * CoglMaterialLayerType:
@@ -778,8 +747,7 @@ typedef enum {
  *
  * Return value: the type of the layer
  */
-CoglMaterialLayerType
-cogl_material_layer_get_type (CoglHandle layer);
+CoglMaterialLayerType cogl_material_layer_get_type (CoglHandle layer);
 
 /**
  * cogl_material_layer_get_texture:
@@ -795,8 +763,7 @@ cogl_material_layer_get_type (CoglHandle layer);
  *
  * Return value: a #CoglHandle for the texture inside the layer
  */
-CoglHandle
-cogl_material_layer_get_texture (CoglHandle layer);
+CoglHandle cogl_material_layer_get_texture (CoglHandle layer);
 
 /**
  * cogl_material_layer_get_min_filter:
@@ -806,8 +773,7 @@ cogl_material_layer_get_texture (CoglHandle layer);
  *
  * Return value: the current downscaling filter
  */
-CoglMaterialFilter
-cogl_material_layer_get_min_filter (CoglHandle layer);
+CoglMaterialFilter cogl_material_layer_get_min_filter (CoglHandle layer);
 
 /**
  * cogl_material_layer_get_mag_filter:
@@ -817,8 +783,7 @@ cogl_material_layer_get_min_filter (CoglHandle layer);
  *
  * Return value: the current downscaling filter
  */
-CoglMaterialFilter
-cogl_material_layer_get_mag_filter (CoglHandle layer);
+CoglMaterialFilter cogl_material_layer_get_mag_filter (CoglHandle layer);
 
 /**
  * cogl_material_set_layer_filters:
@@ -830,11 +795,10 @@ cogl_material_layer_get_mag_filter (CoglHandle layer);
  * Changes the decimation and interpolation filters used when a texture is
  * drawn at other scales than 100%.
  */
-void
-cogl_material_set_layer_filters (CoglHandle         material,
-                                 int                layer_index,
-                                 CoglMaterialFilter min_filter,
-                                 CoglMaterialFilter mag_filter);
+void cogl_material_set_layer_filters (CoglHandle         material,
+                                      gint               layer_index,
+                                      CoglMaterialFilter min_filter,
+                                      CoglMaterialFilter mag_filter);
 
 G_END_DECLS
 
