@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
- *
- *
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors:
  *  Matthew Allum  <mallum@openedhand.com>
@@ -47,31 +47,13 @@
 
 #include "cogl-gles2-wrapper.h"
 
+
 void
-_cogl_texture_driver_gen (GLenum   gl_target,
-                          GLsizei  n,
-                          GLuint  *textures)
+_cogl_texture_driver_bind (GLenum gl_target,
+                           GLuint gl_handle,
+                           GLenum gl_intformat)
 {
-  unsigned int i;
-
-  GE (glGenTextures (n, textures));
-
-  for (i = 0; i < n; i++)
-    {
-      GE (glBindTexture (gl_target, textures[i]));
-
-      switch (gl_target)
-        {
-      case GL_TEXTURE_2D:
-        /* GL_TEXTURE_MAG_FILTER defaults to GL_LINEAR, no need to set it */
-        GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR) );
-        break;
-
-      default:
-        g_assert_not_reached();
-        }
-
-    }
+  GE (cogl_gles2_wrapper_bind_texture (gl_target, gl_handle, gl_intformat));
 }
 
 void
@@ -89,8 +71,7 @@ _cogl_texture_driver_prep_gl_for_pixels_download (int pixels_rowstride,
 }
 
 void
-_cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
-                                             GLuint       gl_handle,
+_cogl_texture_driver_upload_subregion_to_gl (CoglTexture *tex,
                                              int          src_x,
                                              int          src_y,
                                              int          dst_x,
@@ -99,7 +80,8 @@ _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
                                              int          height,
                                              CoglBitmap  *source_bmp,
 				             GLuint       source_gl_format,
-				             GLuint       source_gl_type)
+				             GLuint       source_gl_type,
+                                             GLuint       gl_handle)
 {
   int bpp = _cogl_get_format_bpp (source_bmp->format);
   CoglBitmap slice_bmp;
@@ -112,11 +94,11 @@ _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
    * rowstride = bpp * width and the texture image is not sliced */
 
   /* Setup temp bitmap for slice subregion */
-  slice_bmp.format = source_bmp->format;
+  slice_bmp.format = tex->bitmap.format;
   slice_bmp.width  = width;
   slice_bmp.height = height;
   slice_bmp.rowstride = bpp * slice_bmp.width;
-  slice_bmp.data = g_malloc (slice_bmp.rowstride * slice_bmp.height);
+  slice_bmp.data = (guchar*) g_malloc (slice_bmp.rowstride * slice_bmp.height);
 
   /* Setup gl alignment to match rowstride and top-left corner */
   _cogl_texture_driver_prep_gl_for_pixels_upload (slice_bmp.rowstride,
@@ -131,9 +113,11 @@ _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
                                slice_bmp.width,
                                slice_bmp.height);
 
-  GE( glBindTexture (gl_target, gl_handle) );
+  /* Upload new image data */
+  GE( _cogl_texture_driver_bind (tex->gl_target,
+                                 gl_handle, tex->gl_intformat) );
 
-  GE( glTexSubImage2D (gl_target, 0,
+  GE( glTexSubImage2D (tex->gl_target, 0,
                        dst_x, dst_y,
                        width, height,
                        source_gl_format,
@@ -144,51 +128,6 @@ _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
   g_free (slice_bmp.data);
 }
 
-void
-_cogl_texture_driver_upload_to_gl (GLenum       gl_target,
-                                   GLuint       gl_handle,
-                                   CoglBitmap  *source_bmp,
-                                   GLint        internal_gl_format,
-                                   GLuint       source_gl_format,
-                                   GLuint       source_gl_type)
-{
-  int bpp = _cogl_get_format_bpp (source_bmp->format);
-  CoglBitmap bmp = *source_bmp;
-  gboolean bmp_owner = FALSE;
-
-  /* If the rowstride can't be specified with just GL_ALIGNMENT alone
-     then we need to copy the bitmap because there is no GL_ROW_LENGTH */
-  if (source_bmp->rowstride / bpp != source_bmp->width)
-    {
-      bmp.rowstride = bpp * bmp.width;
-      bmp.data = g_malloc (bmp.rowstride * bmp.height);
-      bmp_owner = TRUE;
-
-      _cogl_bitmap_copy_subregion (source_bmp,
-                                   &bmp,
-                                   0, 0, 0, 0,
-                                   bmp.width,
-                                   bmp.height);
-    }
-
-  /* Setup gl alignment to match rowstride and top-left corner */
-  _cogl_texture_driver_prep_gl_for_pixels_upload (bmp.rowstride,
-                                                  bpp);
-
-  GE( glBindTexture (gl_target, gl_handle) );
-
-  GE( glTexImage2D (gl_target, 0,
-                    internal_gl_format,
-                    bmp.width, bmp.height,
-                    0,
-                    source_gl_format,
-                    source_gl_type,
-                    bmp.data) );
-
-  if (bmp_owner)
-    g_free (bmp.data);
-}
-
 /* NB: GLES doesn't support glGetTexImage2D, so cogl-texture will instead
  * fallback to a generic render + readpixels approach to downloading
  * texture data. (See _cogl_texture_draw_and_read() ) */
@@ -322,7 +261,7 @@ _cogl_texture_driver_allows_foreign_gl_target (GLenum gl_target)
 void
 _cogl_texture_driver_gl_generate_mipmaps (GLenum gl_target)
 {
-  GE( _cogl_wrap_glGenerateMipmap (gl_target) );
+  GE( cogl_wrap_glGenerateMipmap (gl_target) );
 }
 
 CoglPixelFormat
