@@ -1,10 +1,7 @@
 /* Clutter.
  * An OpenGL based 'interactive canvas' library.
- *
  * Authored By Matthew Allum  <mallum@openedhand.com>
- *
- * Copyright (C) 2006, 2007, 2008 OpenedHand
- * Copyright (C) 2009, 2010 Intel Corp
+ * Copyright (C) 2006-2007 OpenedHand
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,9 +14,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
- *
- *
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifndef __CLUTTER_BACKEND_EGL_H__
@@ -41,6 +38,7 @@ G_BEGIN_DECLS
 typedef struct _ClutterBackendEGL       ClutterBackendEGL;
 typedef struct _ClutterBackendEGLClass  ClutterBackendEGLClass;
 
+
 struct _ClutterBackendEGL
 {
   ClutterBackend parent_instance;
@@ -50,15 +48,11 @@ struct _ClutterBackendEGL
   EGLSurface egl_surface;
   EGLContext egl_context;
 
-  /* from the backend */
-  gint surface_width;
-  gint surface_height;
-
-  gint egl_version_major;
-  gint egl_version_minor;
+  gint       egl_version_major;
+  gint       egl_version_minor;
 
   /* main stage singleton */
-  ClutterStageWindow *stage;
+  ClutterActor *stage;
 
   /* event source */
   GSource *event_source;
@@ -66,9 +60,6 @@ struct _ClutterBackendEGL
   /* event timer */
   GTimer *event_timer;
 
-  /* FB device */
-  gint fb_device_id;
-
   /*< private >*/
 };
 
@@ -79,8 +70,8 @@ struct _ClutterBackendEGLClass
 
 GType clutter_backend_egl_get_type (void) G_GNUC_CONST;
 
-void _clutter_events_egl_init   (ClutterBackendEGL *backend);
-void _clutter_events_egl_uninit (ClutterBackendEGL *backend);
+void _clutter_events_init (ClutterBackend *backend);
+void _clutter_events_uninit (ClutterBackend *backend);
 
 G_END_DECLS
 
