@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
- *
- *
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
@@ -44,7 +44,6 @@
 #include "clutter-x11-texture-pixmap.h"
 #include "clutter-x11.h"
 #include "clutter-backend-x11.h"
-#include "clutter-private.h"
 
 #include "cogl/cogl.h"
 
@@ -75,7 +74,6 @@ enum
 enum
 {
   UPDATE_AREA,
-  QUEUE_DAMAGE_REDRAW,
   /* FIXME: Pixmap lost signal? */
   LAST_SIGNAL
 };
@@ -215,41 +213,37 @@ try_alloc_shm (ClutterX11TexturePixmap *texture)
   priv->shminfo.shmid = shmget (IPC_PRIVATE,
 				dummy_image->bytes_per_line
 				* dummy_image->height,
-				IPC_CREAT | 0777);
+				IPC_CREAT|0777);
   if (priv->shminfo.shmid == -1)
     goto failed_shmget;
 
-  priv->shminfo.shmaddr = shmat (priv->shminfo.shmid, 0, 0);
-  if (priv->shminfo.shmaddr == (void *) -1)
+  priv->shminfo.shmaddr =
+    shmat (priv->shminfo.shmid, 0, 0);
+  if (priv->shminfo.shmaddr == (void *)-1)
     goto failed_shmat;
 
   priv->shminfo.readOnly = False;
 
-  if (XShmAttach (dpy, &priv->shminfo) == 0)
+  if (XShmAttach(dpy, &priv->shminfo) == 0)
     goto failed_xshmattach;
 
   if (clutter_x11_untrap_x_errors ())
     g_warning ("X Error: Failed to setup XShm");
 
-  XDestroyImage (dummy_image);
-
   priv->have_shm = TRUE;
-
   return TRUE;
 
 failed_xshmattach:
   g_warning ("XShmAttach failed");
-  shmdt (priv->shminfo.shmaddr);
-
+  shmdt(priv->shminfo.shmaddr);
 failed_shmat:
   g_warning ("shmat failed");
-  shmctl (priv->shminfo.shmid, IPC_RMID, 0);
-
+  shmctl(priv->shminfo.shmid, IPC_RMID, 0);
 failed_shmget:
   g_warning ("shmget failed");
-  XDestroyImage (dummy_image);
-
+  XDestroyImage(dummy_image);
 failed_image_create:
+
   if (clutter_x11_untrap_x_errors ())
     g_warning ("X Error: Failed to setup XShm");
 
@@ -257,68 +251,60 @@ failed_image_create:
   return FALSE;
 }
 
-static void
-check_for_pixmap_damage (ClutterX11TexturePixmap *texture)
-{
-  ClutterX11TexturePixmapPrivate *priv = texture->priv;
-  Display *dpy;
-  XserverRegion parts;
-  int i, r_count;
-  XRectangle *r_damage;
-  XRectangle r_bounds;
-
-  clutter_x11_trap_x_errors ();
-
-  /*
-   * Retrieve the damaged region and break it down into individual
-   * rectangles so we do not have to update the whole shebang.
-   */
-  dpy = clutter_x11_get_default_display();
-  parts = XFixesCreateRegion (dpy, 0, 0);
-  XDamageSubtract (dpy, priv->damage, None, parts);
-
-  r_damage = XFixesFetchRegionAndBounds (dpy,
-                                         parts,
-                                         &r_count,
-                                         &r_bounds);
-
-  clutter_x11_untrap_x_errors ();
-
-  if (r_damage)
-    {
-      for (i = 0; i < r_count; ++i)
-        clutter_x11_texture_pixmap_update_area (texture,
-                                                r_damage[i].x,
-                                                r_damage[i].y,
-                                                r_damage[i].width,
-                                                r_damage[i].height);
-      XFree (r_damage);
-    }
-
-  XFixesDestroyRegion (dpy, parts);
-}
-
 static ClutterX11FilterReturn
 on_x_event_filter (XEvent *xev, ClutterEvent *cev, gpointer data)
 {
   ClutterX11TexturePixmap        *texture;
   ClutterX11TexturePixmapPrivate *priv;
+  Display                        *dpy;
 
   texture = CLUTTER_X11_TEXTURE_PIXMAP (data);
 
   g_return_val_if_fail (CLUTTER_X11_IS_TEXTURE_PIXMAP (texture), \
                         CLUTTER_X11_FILTER_CONTINUE);
 
+  dpy = clutter_x11_get_default_display();
   priv = texture->priv;
 
   if (xev->type == _damage_event_base + XDamageNotify)
     {
+      XserverRegion  parts;
+      gint           i, r_count;
+      XRectangle    *r_damage;
+      XRectangle     r_bounds;
       XDamageNotifyEvent *dev = (XDamageNotifyEvent*)xev;
 
       if (dev->drawable != priv->damage_drawable)
         return CLUTTER_X11_FILTER_CONTINUE;
 
-      check_for_pixmap_damage (texture);
+
+      clutter_x11_trap_x_errors ();
+      /*
+       * Retrieve the damaged region and break it down into individual
+       * rectangles so we do not have to update the whole shebang.
+       */
+      parts = XFixesCreateRegion (dpy, 0, 0);
+      XDamageSubtract (dpy, priv->damage, None, parts);
+
+      r_damage = XFixesFetchRegionAndBounds (dpy,
+                                             parts,
+                                             &r_count,
+                                             &r_bounds);
+
+      clutter_x11_untrap_x_errors ();
+
+      if (r_damage)
+        {
+          for (i = 0; i < r_count; ++i)
+            clutter_x11_texture_pixmap_update_area (texture,
+                                                    r_damage[i].x,
+                                                    r_damage[i].y,
+                                                    r_damage[i].width,
+                                                    r_damage[i].height);
+          XFree (r_damage);
+        }
+
+      XFixesDestroyRegion (dpy, parts);
     }
 
   return  CLUTTER_X11_FILTER_CONTINUE;
@@ -414,54 +400,6 @@ free_damage_resources (ClutterX11TexturePixmap *texture)
     }
 }
 
-static void
-clutter_x11_texture_pixmap_real_queue_damage_redraw (
-                                              ClutterX11TexturePixmap *texture,
-                                              gint x,
-                                              gint y,
-                                              gint width,
-                                              gint height)
-{
-  ClutterActor    *self = CLUTTER_ACTOR (texture);
-  ClutterActorBox  allocation;
-  guint	           pixmap_width = 0;
-  guint            pixmap_height = 0;
-  float            scale_x;
-  float            scale_y;
-  ClutterActorBox  clip;
-
-  /* NB: clutter_actor_queue_clipped_redraw expects a box in the actor's
-   * coordinate space so we need to convert from pixmap coordinates to
-   * actor coordinates...
-   */
-
-  /* XXX: we don't care if we get an out of date allocation here because
-   * clutter_actor_queue_clipped_redraw knows to ignore the clip if the
-   * actor's allocation is invalid.
-   *
-   * This is noted because clutter_actor_get_allocation_box does some
-   * unnecessary work to support buggy code with a comment suggesting that
-   * it could be changed later which would be good for this use case!
-   */
-  clutter_actor_get_allocation_box (self, &allocation);
-
-  g_object_get (self,
-                "pixmap-width",  &pixmap_width,
-                "pixmap-height", &pixmap_height,
-                NULL);
-
-  scale_x = (allocation.x2 - allocation.x1) / pixmap_width;
-  scale_y = (allocation.y2 - allocation.y1) / pixmap_height;
-
-  clip.x1 = x * scale_x;
-  clip.y1 = y * scale_y;
-  clip.x2 =  clip.x1 + width * scale_x;
-  clip.y2 = clip.y1 + height * scale_y;
-
-  _clutter_actor_queue_redraw_with_clip (self,
-                                         CLUTTER_REDRAW_CLIPPED_TO_BOX,
-                                         &clip);
-}
 
 static void
 clutter_x11_texture_pixmap_init (ClutterX11TexturePixmap *self)
@@ -787,44 +725,6 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
                     G_TYPE_INT,
                     G_TYPE_INT);
 
-  /**
-   * ClutterX11TexturePixmap::queue-damage-redraw
-   * @texture: the object which received the signal
-   * @x: The top left x position of the damage region
-   * @y: The top left y position of the damage region
-   * @width: The width of the damage region
-   * @height: The height of the damage region
-   *
-   * ::queue-damage-redraw is emitted to notify that some sub-region of the
-   * underlying pixmap has changed and you need to queue a
-   * corresponding redraw for the actor.
-   *
-   * The default handler will queue a clipped redraw in response to
-   * the damage, using the assumption that the pixmap is being painted
-   * to a rectangle covering the transformed allocation of the actor.
-   * If you sub-class and change the paint method so this isn't true
-   * then you must also provide your own damage signal handler to
-   * queue a redraw that blocks this default behaviour.
-   *
-   * Since: 1.2
-   */
-  signals[QUEUE_DAMAGE_REDRAW] =
-    g_signal_new (g_intern_static_string ("queue-damage-redraw"),
-                  G_TYPE_FROM_CLASS (object_class),
-                  G_SIGNAL_RUN_FIRST,
-                  0,
-                  NULL, NULL,
-                  clutter_marshal_VOID__INT_INT_INT_INT,
-                  G_TYPE_NONE, 4,
-                  G_TYPE_INT,
-                  G_TYPE_INT,
-                  G_TYPE_INT,
-                  G_TYPE_INT);
-
-  g_signal_override_class_handler ("queue-damage-redraw",
-                                   CLUTTER_X11_TYPE_TEXTURE_PIXMAP,
-                                   G_CALLBACK (clutter_x11_texture_pixmap_real_queue_damage_redraw));
-
   default_backend = clutter_get_default_backend ();
 
   if (!CLUTTER_IS_BACKEND_X11 (default_backend))
@@ -1037,13 +937,10 @@ clutter_x11_texture_pixmap_update_area_real (ClutterX11TexturePixmap *texture,
 /**
  * clutter_x11_texture_pixmap_new:
  *
- * Creates a new #ClutterX11TexturePixmap which can be used to display the
- * contents of an X11 Pixmap inside a Clutter scene graph
- *
  * Return value: A new #ClutterX11TexturePixmap
  *
  * Since: 0.8
- */
+ **/
 ClutterActor *
 clutter_x11_texture_pixmap_new (void)
 {
@@ -1058,12 +955,10 @@ clutter_x11_texture_pixmap_new (void)
  * clutter_x11_texture_pixmap_new_with_pixmap:
  * @pixmap: the X Pixmap to which this texture should be bound
  *
- * Creates a new #ClutterX11TexturePixmap for @pixmap
- *
  * Return value: A new #ClutterX11TexturePixmap bound to the given X Pixmap
  *
  * Since: 0.8
- */
+ **/
 ClutterActor *
 clutter_x11_texture_pixmap_new_with_pixmap (Pixmap pixmap)
 {
@@ -1080,8 +975,6 @@ clutter_x11_texture_pixmap_new_with_pixmap (Pixmap pixmap)
  * clutter_x11_texture_pixmap_new_with_window:
  * @window: the X window to which this texture should be bound
  *
- * Creates a new #ClutterX11TexturePixmap for @window
- *
  * Return value: A new #ClutterX11TexturePixmap bound to the given X window.
  *
  * Since: 0.8
@@ -1106,7 +999,7 @@ clutter_x11_texture_pixmap_new_with_window (Window window)
  * Sets the X Pixmap to which the texture should be bound.
  *
  * Since: 0.8
- */
+ **/
 void
 clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
                                        Pixmap                   pixmap)
@@ -1228,17 +1121,16 @@ clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
  * clutter_x11_texture_pixmap_set_window:
  * @texture: the texture to bind
  * @window: the X window to which the texture should be bound
- * @automatic: %TRUE for automatic window updates, %FALSE for manual.
+ * @automatic: TRUE is automatic window updates, FALSE for manual.
  *
  * Sets up a suitable pixmap for the window, using the composite and damage
  * extensions if possible, and then calls
- * clutter_x11_texture_pixmap_set_pixmap().
- *
- * If you want to display a window in a #ClutterTexture, you probably want
- * this function, or its older sister, clutter_glx_texture_pixmap_set_window()
+ * clutter_x11_texture_pixmap_set_pixmap(). If you want a window in a texture,
+ * you probably want this function, or its older sister,
+ * clutter_glx_texture_pixmap_set_window().
  *
  * Since: 0.8
- */
+ **/
 void
 clutter_x11_texture_pixmap_set_window (ClutterX11TexturePixmap *texture,
                                        Window                   window,
@@ -1330,7 +1222,7 @@ clutter_x11_texture_pixmap_set_window (ClutterX11TexturePixmap *texture,
  * pixmap's invalidation as the window changed size.
  *
  * Since: 0.8
- */
+ **/
 void
 clutter_x11_texture_pixmap_sync_window (ClutterX11TexturePixmap *texture)
 {
@@ -1465,24 +1357,8 @@ clutter_x11_texture_pixmap_update_area (ClutterX11TexturePixmap *texture,
   g_return_if_fail (CLUTTER_X11_IS_TEXTURE_PIXMAP (texture));
 
   g_signal_emit (texture, signals[UPDATE_AREA], 0, x, y, width, height);
-
-  /* The default handler for the "queue-damage-redraw" signal is
-   * clutter_x11_texture_pixmap_real_queue_damage_redraw which will queue a
-   * clipped redraw. */
-  g_signal_emit (texture, signals[QUEUE_DAMAGE_REDRAW],
-                 0, x, y, width, height);
 }
 
-/**
- * clutter_x11_texture_pixmap_set_automatic:
- * @texture: a #ClutterX11TexturePixmap
- * @setting: %TRUE to enable automatic updates
- *
- * Enables or disables the automatic updates ot @texture in case the backing
- * pixmap or window is damaged
- *
- * Since: 0.8
- */
 void
 clutter_x11_texture_pixmap_set_automatic (ClutterX11TexturePixmap *texture,
                                           gboolean                 setting)
@@ -1507,4 +1383,3 @@ clutter_x11_texture_pixmap_set_automatic (ClutterX11TexturePixmap *texture,
   priv->automatic_updates = setting;
 
 }
-
